"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HasuraBackgroundClient = void 0;
const child_process_1 = require("child_process");
const util_1 = __importDefault(require("@cardano-graphql/util"));
const graphql_request_1 = require("graphql-request");
const p_retry_1 = __importDefault(require("p-retry"));
const path_1 = __importDefault(require("path"));
const ts_log_1 = require("ts-log");
const epochInformationNotYetAvailable = 'Epoch information not yet available. This is expected during the initial chain-sync.';
const withHexPrefix = (value) => `\\x${value !== undefined ? value : ''}`;
class HasuraBackgroundClient {
    constructor(hasuraCliPath, hasuraUri, logger = ts_log_1.dummyLogger) {
        this.hasuraCliPath = hasuraCliPath;
        this.hasuraUri = hasuraUri;
        this.logger = logger;
        this.state = null;
        this.applyingSchemaAndMetadata = false;
        this.client = new graphql_request_1.GraphQLClient(`${this.hasuraUri}/v1/graphql`, {
            headers: {
                'X-Hasura-Role': 'cardano-graphql'
            }
        });
    }
    hasuraCli(command) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                child_process_1.exec(`${this.hasuraCliPath} --skip-update-check --project ${path_1.default.resolve(__dirname, '..', 'hasura', 'project')} --endpoint ${this.hasuraUri} ${command}`, (error, stdout) => {
                    if (error) {
                        reject(error);
                    }
                    if (stdout !== '')
                        this.logger.debug({ module: 'HasuraBackgroundClient' }, stdout);
                    resolve();
                });
            });
        });
    }
    initialize() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.state !== null)
                return;
            this.state = 'initializing';
            this.logger.info({ module: 'HasuraBackgroundClient' }, 'Initializing');
            yield this.applySchemaAndMetadata();
            this.logger.debug({ module: 'HasuraBackgroundClient' }, 'graphql-engine setup');
            yield p_retry_1.default(() => __awaiter(this, void 0, void 0, function* () {
                const result = yield this.client.request(graphql_request_1.gql `query {
            epochs (limit: 1, order_by: { number: desc }) {
                number
            }
        }`);
                if (result.epochs.length === 0) {
                    this.logger.debug({ module: 'HasuraBackgroundClient' }, epochInformationNotYetAvailable);
                    throw new Error(epochInformationNotYetAvailable);
                }
            }), {
                factor: 1.05,
                retries: 100,
                onFailedAttempt: util_1.default.onFailedAttemptFor('Detecting DB sync state has reached minimum progress', this.logger)
            });
            this.logger.debug({ module: 'HasuraBackgroundClient' }, 'DB sync state has reached minimum progress');
            this.state = 'initialized';
            this.logger.info({ module: 'HasuraBackgroundClient' }, 'Initialized');
        });
    }
    shutdown() {
        return __awaiter(this, void 0, void 0, function* () {
            this.state = null;
        });
    }
    applySchemaAndMetadata() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.applyingSchemaAndMetadata)
                return;
            this.applyingSchemaAndMetadata = true;
            yield p_retry_1.default(() => __awaiter(this, void 0, void 0, function* () {
                yield this.hasuraCli('migrate --database-name default apply --down all');
                yield this.hasuraCli('migrate --database-name default apply --up all');
            }), {
                factor: 1.75,
                retries: 9,
                onFailedAttempt: util_1.default.onFailedAttemptFor('Applying PostgreSQL schema migrations', this.logger)
            });
            yield p_retry_1.default(() => __awaiter(this, void 0, void 0, function* () {
                yield this.hasuraCli('metadata clear');
                yield this.hasuraCli('metadata apply');
            }), {
                factor: 1.75,
                retries: 9,
                onFailedAttempt: util_1.default.onFailedAttemptFor('Applying Hasura metadata', this.logger)
            });
            this.applyingSchemaAndMetadata = false;
        });
    }
    deleteAssetsAfterSlot(slotNo) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.debug({ module: 'HasuraClient', slotNo }, 'deleting assets found in tokens after slot');
            const result = yield this.client.request(graphql_request_1.gql `mutation DeleteAssetsAfterSlot($slotNo: Int!) {
          delete_assets(
              where: {
                  firstAppearedInSlot: {
                      _gt: $slotNo
                  }
              }
          ) {
              affected_rows
          }
      }`, {
                slotNo
            });
            return result.delete_assets.affected_rows;
        });
    }
    hasAsset(assetId) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.client.request(graphql_request_1.gql `query HasAsset (
          $assetId: bytea!
      ) {
          assets (
              where: { assetId: { _eq: $assetId }}
          ) {
              assetId
          }
      }`, {
                assetId: withHexPrefix(assetId)
            });
            const response = result.assets.length > 0;
            this.logger.debug({ module: 'HasuraClient', assetId, hasAsset: response }, 'Has asset?');
            return response;
        });
    }
    getMostRecentPointWithNewAsset() {
        return __awaiter(this, void 0, void 0, function* () {
            let point;
            yield p_retry_1.default(() => __awaiter(this, void 0, void 0, function* () {
                const result = yield this.client.request(graphql_request_1.gql `query {
            assets (
                limit: 1
                offset: 1
                order_by: { firstAppearedInBlock: { slotNo: desc }}
            ) {
                firstAppearedInBlock {
                    hash
                    slotNo
                }
            }
        }`);
                if (result.errors !== undefined) {
                    throw new Error(result.errors);
                }
                if (result.assets.length !== 0) {
                    if (result.assets[0].firstAppearedInBlock === null) {
                        throw new Error('cardano-db-sync is lagging behind the asset sync operation.');
                    }
                    const { hash, slotNo } = result.assets[0].firstAppearedInBlock;
                    point = {
                        hash: hash.substring(2),
                        slot: Number(slotNo)
                    };
                }
                else {
                    point = null;
                }
            }), {
                factor: 1.5,
                retries: 1000,
                onFailedAttempt: util_1.default.onFailedAttemptFor('Getting the most recent point with a new asset', this.logger)
            });
            return point;
        });
    }
    addAssetMetadata(asset) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.info({ module: 'HasuraClient', assetId: asset.assetId }, 'Adding metadata to asset');
            const result = yield this.client.request(graphql_request_1.gql `mutation AddAssetMetadata(
          $assetId: bytea!
          $decimals: Int
          $description: String
          $logo: String
          $metadataHash: bpchar!
          $name: String
          $ticker: String
          $url: String
      ) {
          update_assets(
              where: {
                  assetId: { _eq: $assetId }
              },
              _set: {
                  decimals: $decimals
                  description: $description
                  logo: $logo
                  metadataHash: $metadataHash
                  name: $name
                  ticker: $ticker
                  url: $url
              }
          ) {
              affected_rows
              returning {
                  assetId
              }
          }
      }`, Object.assign(Object.assign({}, asset), { assetId: withHexPrefix(asset.assetId) }));
            if (result.errors !== undefined) {
                throw new Error(result.errors);
            }
        });
    }
    insertAssets(assets) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.debug({ module: 'HasuraClient', qty: assets.length }, 'inserting assets found in tokens');
            const result = yield this.client.request(graphql_request_1.gql `mutation InsertAssets($assets: [Asset_insert_input!]!) {
          insert_assets(objects: $assets) {
              returning {
                  name
                  policyId
                  description
                  assetName
                  assetId
              }
              affected_rows
          }
      }`, {
                assets: assets.map(asset => (Object.assign(Object.assign({}, asset), {
                    assetId: withHexPrefix(asset.assetId),
                    assetName: withHexPrefix(asset.assetName),
                    policyId: withHexPrefix(asset.policyId)
                })))
            });
            return result;
        });
    }
    getAssetMetadataHashesById(assetIds) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.client.request(graphql_request_1.gql `query AssetMetadataHashes (
          $assetIds: [bytea!]!
      ){
          assets (
              where: {
                  assetId: { _in: $assetIds }
              }) {
              assetId
              metadataHash
          }
      }`, {
                assetIds: assetIds.map(id => withHexPrefix(id))
            });
            return result.assets;
        });
    }
}
exports.HasuraBackgroundClient = HasuraBackgroundClient;
//# sourceMappingURL=HasuraBackgroundClient.js.map