"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MissingConfig = void 0;
const bunyan_1 = require("bunyan");
const index_1 = require("./index");
const death_1 = __importDefault(require("death"));
const ts_custom_error_1 = require("ts-custom-error");
const fs_extra_1 = __importDefault(require("fs-extra"));
class MissingConfig extends ts_custom_error_1.CustomError {
    constructor(message) {
        super();
        this.message = message;
    }
}
exports.MissingConfig = MissingConfig;
function getConfig() {
    return __awaiter(this, void 0, void 0, function* () {
        const env = filterAndTypecastEnvs(process.env);
        if (!env.hasuraCliPath) {
            throw new MissingConfig('HASURA_CLI_PATH env not set');
        }
        if (!env.hasuraUri) {
            throw new MissingConfig('HASURA_URI env not set');
        }
        if (!env.metadataServerUri) {
            throw new MissingConfig('METADATA_SERVER_URI env not set');
        }
        if (!env.postgres.dbFile && !env.postgres.db) {
            throw new MissingConfig('POSTGRES_DB_FILE or POSTGRES_DB env not set');
        }
        if (!env.postgres.host) {
            throw new MissingConfig('POSTGRES_HOST env not set');
        }
        if (!env.postgres.passwordFile && !env.postgres.password) {
            throw new MissingConfig('POSTGRES_PASSWORD_FILE or POSTGRES_PASSWORD env not set');
        }
        if (!env.postgres.port) {
            throw new MissingConfig('POSTGRES_PORT env not set');
        }
        if (!env.postgres.userFile && !env.postgres.user) {
            throw new MissingConfig('POSTGRES_USER_FILE or POSTGRES_USER env not set');
        }
        let db;
        try {
            db = {
                database: env.postgres.db || (yield fs_extra_1.default.readFile(env.postgres.dbFile, 'utf8')).toString().trim(),
                host: env.postgres.host,
                password: env.postgres.password || (yield fs_extra_1.default.readFile(env.postgres.passwordFile, 'utf8')).toString().trim(),
                port: env.postgres.port,
                user: env.postgres.user || (yield fs_extra_1.default.readFile(env.postgres.userFile, 'utf8')).toString().trim()
            };
        }
        catch (error) {
            throw new MissingConfig('Database configuration cannot be read');
        }
        const { postgres } = env, selectedEnv = __rest(env, ["postgres"]);
        return Object.assign(Object.assign({}, selectedEnv), { db, loggerMinSeverity: env.loggerMinSeverity || 'info' });
    });
}
function filterAndTypecastEnvs(env) {
    const { ASSET_METADATA_UPDATE_INTERVAL, HASURA_CLI_PATH, HASURA_URI, LOGGER_MIN_SEVERITY, METADATA_SERVER_URI, OGMIOS_HOST, OGMIOS_PORT, POSTGRES_DB, POSTGRES_DB_FILE, POSTGRES_HOST, POSTGRES_PASSWORD, POSTGRES_PASSWORD_FILE, POSTGRES_PORT, POSTGRES_USER, POSTGRES_USER_FILE } = env;
    return {
        hasuraCliPath: HASURA_CLI_PATH,
        hasuraUri: HASURA_URI,
        loggerMinSeverity: LOGGER_MIN_SEVERITY,
        metadataServerUri: METADATA_SERVER_URI,
        metadataUpdateInterval: {
            assets: ASSET_METADATA_UPDATE_INTERVAL ? Number(ASSET_METADATA_UPDATE_INTERVAL) : undefined
        },
        ogmios: {
            host: OGMIOS_HOST,
            port: OGMIOS_PORT ? Number(OGMIOS_PORT) : undefined
        },
        postgres: {
            db: POSTGRES_DB,
            dbFile: POSTGRES_DB_FILE,
            host: POSTGRES_HOST,
            password: POSTGRES_PASSWORD,
            passwordFile: POSTGRES_PASSWORD_FILE,
            port: POSTGRES_PORT ? Number(POSTGRES_PORT) : undefined,
            user: POSTGRES_USER,
            userFile: POSTGRES_USER_FILE
        }
    };
}
(function () {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const config = yield getConfig();
        const logger = bunyan_1.createLogger({
            name: 'background',
            level: config.loggerMinSeverity
        });
        try {
            const hasuraBackgroundClient = new index_1.HasuraBackgroundClient(config.hasuraCliPath, config.hasuraUri, logger);
            const chainFollower = new index_1.ChainFollower(hasuraBackgroundClient, logger, config.db);
            const metadataClient = new index_1.MetadataClient(config.metadataServerUri, logger);
            const worker = new index_1.Worker(hasuraBackgroundClient, logger, metadataClient, config.db, {
                metadataUpdateInterval: {
                    assets: (_a = config.metadataUpdateInterval) === null || _a === void 0 ? void 0 : _a.assets
                }
            });
            const db = new index_1.Db(config.db, logger);
            const getChainSyncPoints = () => __awaiter(this, void 0, void 0, function* () {
                const mostRecentPoint = yield hasuraBackgroundClient.getMostRecentPointWithNewAsset();
                return mostRecentPoint !== null ? [mostRecentPoint, 'origin'] : ['origin'];
            });
            yield db.init({
                onDbInit: () => hasuraBackgroundClient.shutdown(),
                onDbSetup: () => __awaiter(this, void 0, void 0, function* () {
                    try {
                        yield hasuraBackgroundClient.initialize();
                        yield metadataClient.initialize();
                        yield chainFollower.initialize(config.ogmios, getChainSyncPoints);
                        yield worker.start();
                        yield chainFollower.start(yield getChainSyncPoints());
                    }
                    catch (error) {
                        logger.error(error.message);
                        process.exit(1);
                    }
                })
            });
            death_1.default(() => __awaiter(this, void 0, void 0, function* () {
                yield Promise.all([
                    hasuraBackgroundClient.shutdown,
                    worker.shutdown,
                    chainFollower.shutdown,
                    db.shutdown
                ]);
                process.exit(1);
            }));
        }
        catch (error) {
            logger.error('Exiting due to uncaught exception', error.message);
            process.exit(1);
        }
    });
})();
//# sourceMappingURL=background.js.map